"""
=========
Associate
=========

Widgets for association rules.

"""

# Category description for the widget registry

NAME = "X-Ray Server"

DESCRIPTION = "Widgets for X-Ray Server "

BACKGROUND = "#FDFFE6"

ICON = "icons/xrayserver.png"

PRIORITY = 3001
